package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.general.exception.DescribeException;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.general.domain.enums.common.EwbConstants;
import com.nationrel.weibull.domain.bo.EwbCalcParamBo;
import com.nationrel.weibull.domain.entity.EwbAnalysisSetting;
import com.nationrel.weibull.domain.enums.EwbConfidenceTypeCodeEnum;
import com.nationrel.weibull.domain.enums.EwbDistributingCodeEnum;
import com.nationrel.weibull.domain.enums.EwbParameterCodeEnum;
import com.nationrel.weibull.domain.vo.EwbAnalysisStartVo;
import com.nationrel.weibull.domain.vo.EwbAnalysisStressStrengthVo;
import com.nationrel.weibull.domain.vo.EwbProbabilityPlotVo;
import com.nationrel.weibull.domain.vo.EwbReliabilityPlotVo;
import com.nationrel.weibull.service.EwbCalcService;
import com.nationrel.weibull.service.EwbPlotService;
import com.nationrel.weibull.service.EwbValidatorService;
import com.nationrel.weibull.util.WeibullNumberUtil;
import com.nationrel.weibull.util.ValidateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;
import java.util.Map;

/**
 * @author Aimee on 2020/10/19.
 */
@Api(tags = "寿命数据分析-画图部分")
@Validated
@RestController
@RequestMapping("/chart")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbChartController {

    private final EwbPlotService ewbPlotService;
    private final EwbCalcService ewbCalcService;
    private final EwbValidatorService ewbValidatorService;

    @ApiOperation(value = "获取计算器设置", notes = "分析数据")
    @GetMapping("/calc-setting")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "settingId", value = "设置ID", dataType = "String", paramType = "query")
    })
    public ServerResponse calcSetting(HttpServletRequest request, @NotBlank(message = "SetIdNotEmpty") String settingId, @NotBlank(message = "AnalysisIdNotEmpty") String analysisId) {
        Map<String, Object> resultMap = ewbCalcService.calcSetting(UserInfoUtil.parseHeader(request).getUserId(), settingId, analysisId);
        return ServerResponse.createBySuccess(resultMap);
    }

    @ApiOperation(value = "更新画图的标记以及xy区间信息", notes = "更新画图的标记以及xy区间信息")
    @PostMapping("/markInfo")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse updateMarkInfo(HttpServletRequest request, @RequestBody EwbAnalysisSetting param) {
        return ewbPlotService.updateMarkInfo(param, UserInfoUtil.parseHeader(request).getLanguageKey());
    }

    @ApiOperation(value = "更新分布概率图的图形的名称", notes = "更新分布概率图的图形的名称")
    @PutMapping("/reliabilityPlotName")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse updateReliabilityPlotName(@RequestBody EwbAnalysisSetting param) {
        return ewbPlotService.updateReliabilityPlotName(param);
    }

    @ApiOperation(value = "获取计算结果", notes = "分析数据")
    @GetMapping("/calcResult")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "settingId", value = "设置ID", dataType = "String", paramType = "query")
    })
    public ServerResponse calcResult(HttpServletRequest request, @NotBlank(message = "AnalysisIdNotEmpty") String analysisId) {
        return ServerResponse.createBySuccess(ewbCalcService.calcResult(analysisId, UserInfoUtil.parseHeader(request).getLanguageKey()));
    }

    @ApiOperation(value = "开始计算", notes = "分析数据")
    @PostMapping("/calc")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse calc(HttpServletRequest request, @RequestBody EwbCalcParamBo param) throws Exception {
        HeaderUserInfo user = UserInfoUtil.parseHeader(request);
        Integer languageKey = user.getLanguageKey();
        // 计算类型-可靠度
        String distributingCode = param.getDistributingCode();
        String parameterCode = param.getParameterCode();
        String gamma = param.getGamma();
        BigDecimal gammaBd = new BigDecimal(gamma);
        if (EwbConstants.CALC_TYPE_RELIABILITY.equals(param.getCalcType())) {
            // 结束时间校验--分布为Exponential 2p时结束时间应大于等于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex()) && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex()) && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), gammaBd, EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            } else {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-不可靠度
        } else if (EwbConstants.CALC_TYPE_UNRELIABILITY.equals(param.getCalcType())) {
            // 结束时间校验--分布为Exponential 2p时结束时间应大于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex())
                    && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex())
                    && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), gammaBd, EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            } else {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-平均失效时间
        } else if (EwbConstants.CALC_TYPE_MTBF.equals(param.getCalcType())) {
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-失效率
        } else if (EwbConstants.CALC_TYPE_FAILURERATE.equals(param.getCalcType())) {
            // 结束时间校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-参数
        } else if (EwbConstants.CALC_TYPE_PARAMETER.equals(param.getCalcType())) {
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-BX%寿命
        } else if (EwbConstants.CALC_TYPE_BLIFE.equals(param.getCalcType())) {
            // 失效百分比校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.不可靠度, languageKey), param.getFailurePercent(), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-可靠寿命
        } else if (EwbConstants.CALC_TYPE_TIMEATRELIABILITY.equals(param.getCalcType())) {
            // 可靠寿命百分比校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠度, languageKey), param.getFailurePercent(), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);

            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-条件可靠度
        } else if (EwbConstants.CALC_TYPE_CONDITIONALPROBABILITY.equals(param.getCalcType()) || EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(param.getCalcType())) {
            // 开始时间校验
            // Exponential 2p 开始时间应大于等于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex())
                    && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex())
                    && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), gammaBd, EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            } else {
                calcParamCheckZero(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 附加时间校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.附加时间, languageKey), param.getAdditionalTime(), EwbConstants.CALC_RANGE_TACK_TIME_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            // 计算类型-条件不可靠度
        } else if (EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(param.getCalcType()) || EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(param.getCalcType())) {
            // 开始时间校验
            // Exponential 2p 开始时间应大于等于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex())
                    && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex())
                    && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), gammaBd, EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            } else {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 附加时间校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.附加时间, languageKey), param.getAdditionalTime(), EwbConstants.CALC_RANGE_TACK_TIME_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
        }
        return ewbCalcService.calc(user.getUserId(), param, user, false);
    }

    @ApiOperation(value = "概率图", notes = "作图")
    @PostMapping("/probabilityPlot")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse probabilityPlot(HttpServletRequest request, @RequestBody @Valid EwbAnalysisStartVo param) throws Exception {
        EwbProbabilityPlotVo plotVo = ewbPlotService.probabilityPlot(UserInfoUtil.parseHeader(request), param, false);
        return ServerResponse.createBySuccess(plotVo);
    }

//    @ApiOperation(value = "概率图数据校验", notes = "概率图数据校验")
//    @PutMapping("/probabilityPlot")
//    @ApiImplicitParams(value = {
//            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
//    })
//    public ServerResponse probabilityPlotAna(HttpServletRequest request, @RequestBody @Valid EwbAnalysisStartVo param) {
//        return ewbValidatorService.validatorDataDetail(param.getAnalysisId(), param.getDataId(), param.getSettingId(), param.getParameterCode(), param.getDistributingCode(), param.getEstimationMethodCode(), param.getBelievePercent(), UserInfoUtil.parseHeader(request));
//    }

    @ApiOperation(value = "stressStrength曲线图", notes = "作图")
    @PostMapping("/stressStrengthPlot")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse stressStrengthPlot(@RequestBody EwbAnalysisStressStrengthVo param) throws Exception {
        return ewbPlotService.stressStrengthPlot(param);
    }

    @ApiOperation(value = "除概率图外的其他图", notes = "作图")
    @PostMapping("/image")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse otherPlot(HttpServletRequest request, @RequestBody @Valid EwbAnalysisStartVo param) throws Exception {
        EwbReliabilityPlotVo plotVo = ewbPlotService.selectEchartDetail(UserInfoUtil.parseHeader(request), param);
        return ServerResponse.createBySuccess(plotVo);
    }

    /**
     * @param fieldName  字段名
     * @param gamma      Exponential 2p gamma
     * @param fieldValue 字段值
     * @param min        最小值
     * @param max        最大值
     * @Title: timeCheck
     * @Description: 时间参数校验
     */
    private void timeCheck(String fieldName, String fieldValue, BigDecimal gamma, BigDecimal min, BigDecimal max, Integer languageKey) {
        // 不能为空
        if (ValidateUtil.isEmpty(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.不能为空, languageKey);
        } else if (!ValidateUtil.isNumber(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.必须为数字, languageKey);
        }
        // Exponential 2p可靠度结束时间、不可靠度结束时间、条件可靠度开始时间必须大于gamma
        BigDecimal val = WeibullNumberUtil.parseBigDecimal(fieldValue);
        if (ValidateUtil.isPositiveNumber(fieldValue)) {
            if (val.compareTo(gamma) <= 0 || val.compareTo(max) > 0) {
                throw new DescribeException(fieldName, gamma, max, EwbCustomPromptEnum.范围必须是, languageKey);
            }
        } else {
            // 范围
            if (val.compareTo(min) <= 0 || val.compareTo(max) > 0) {
                throw new DescribeException(fieldName, min, max, EwbCustomPromptEnum.范围必须是, languageKey);
            }

        }
    }

    /**
     * @param fieldName  待校验字段名
     * @param fieldValue 待校验字段值
     * @param min        校验范围最小值
     * @param max        校验范围最大值
     * @Title: calcParamCheck
     * @Description: 参数校验
     */
    public static void calcParamCheck(String fieldName, String fieldValue, BigDecimal min, BigDecimal max, Integer languageKey) {

        if (ValidateUtil.isEmpty(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.不能为空, languageKey);
        } else if (!ValidateUtil.isNumber(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.必须为数字, languageKey);
        } else if (WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(min) <= 0 || WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(max) >= 0) {
            throw new DescribeException(fieldName, min, max, EwbCustomPromptEnum.范围必须是, languageKey);
        }
    }

    /**
     * 可为0
     * @param fieldName  待校验字段名
     * @param fieldValue 待校验字段值
     * @param min        校验范围最小值
     * @param max        校验范围最大值
     * @Title: calcParamCheck
     * @Description: 参数校验
     */
    public static void calcParamCheckZero(String fieldName, String fieldValue, BigDecimal min, BigDecimal max, Integer languageKey) {

        if (ValidateUtil.isEmpty(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.不能为空, languageKey);
        } else if (!ValidateUtil.isNumber(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.必须为数字, languageKey);
        } else if (WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(min) < 0 || WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(max) > 0) {
            throw new DescribeException(fieldName, min, max, EwbCustomPromptEnum.范围必须是, languageKey);
        }
    }

}
