package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.weibull.domain.entity.EwbMultiChart;
import com.nationrel.weibull.domain.vo.*;
import com.nationrel.weibull.service.EwbMultiChartService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;

@Api(tags = "寿命数据多图")
@RestController
@RequestMapping("/multichart")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbMultiChartController {

    private final EwbMultiChartService ewbMultiChartService;

    @ApiOperation(value = "多图创建", notes = "多图")
    @PostMapping("/create")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse create(HttpServletRequest request, @RequestBody @Valid EwbMultiChartCreateVo param) {
        return ewbMultiChartService.createMultiChart(UserInfoUtil.parseHeader(request), param);
    }

    @ApiOperation(value = "更新画图的标记以及xy区间信息", notes = "更新画图的标记以及xy区间信息")
    @PostMapping("/markInfo")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse updateMarkInfo(HttpServletRequest request, @RequestBody EwbMultiChart param) {
        return ewbMultiChartService.updateMarkInfo(param, UserInfoUtil.parseHeader(request).getLanguageKey());
    }

    @ApiOperation(value = "多图列表", notes = "多图")
    @GetMapping("/list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse list(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return ServerResponse.createBySuccess(ewbMultiChartService.list(wbId));
    }

    @ApiOperation(value = "多图名称修改-删除(修改名称传多图名称，删除时del_flag=1)", notes = "多图")
    @PostMapping("/edit")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse edit(HttpServletRequest request, @RequestBody EwbMultiChart ewbMultiChart) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        ewbMultiChartService.edit(userInfo, ewbMultiChart);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "分析列表", notes = "多图")
    @GetMapping("/analysis-list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "status", value = "分析类型", dataType = "String", paramType = "query")
    })
    public ServerResponse analysisList(HttpServletRequest request, @RequestParam String wbId, @RequestParam(required = false) String multiChartId, @RequestParam(required = false) String analysisType) {
        if (StringUtils.isBlank(wbId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, UserInfoUtil.parseLk(request));
        }
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        List<EwbMultiChartAnalysisListVo> analysisListVos = ewbMultiChartService.selectAnalysisList(wbId, multiChartId, analysisType, userInfo.getUserId());
        return ServerResponse.createBySuccess(analysisListVos);
    }

    @ApiOperation(value = "分析保存", notes = "多图")
    @PostMapping("/save-list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveList(HttpServletRequest request, @RequestBody EwbMultiChartSaveListVo param) {
        if (StringUtils.isBlank(param.getMultiChartId())) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.多图ID不可以为空, UserInfoUtil.parseLk(request));
        }
        if (StringUtils.isBlank(param.getAnalysisIds())) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.至少选择一个分析不可以为空, UserInfoUtil.parseLk(request));
        }
        ewbMultiChartService.saveList(param, UserInfoUtil.parseLk(request));
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "概率图", notes = "作图")
    @GetMapping("/probabilityPlot")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse probabilityPlot(HttpServletRequest request, @RequestParam String multiChartId) {
        if (StringUtils.isBlank(multiChartId)) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.多图ID不可以为空, UserInfoUtil.parseLk(request));
        }
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        List<EwbProbabilityPlotVo> ewbProbabilityPlotVos = ewbMultiChartService.probabilityPlot(userInfo, multiChartId);
        return ServerResponse.createBySuccess(ewbProbabilityPlotVos);
    }

    @ApiOperation(value = "可靠度", notes = "作图")
    @GetMapping("/image")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse reliabilityPlot(HttpServletRequest request, @RequestParam String multiChartId, @RequestParam String plotType, @RequestParam Double xaxisStart, @RequestParam Double xaxisEnd, @RequestParam Double yaxisStart, @RequestParam Double yaxisEnd) {
        if (StringUtils.isBlank(multiChartId)) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.多图ID不可以为空, UserInfoUtil.parseLk(request));
        }
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        List<EwbProbabilityPlotVo> ewbProbabilityPlotVos = ewbMultiChartService.selectEchartDetail(userInfo, multiChartId, plotType, xaxisStart, xaxisEnd, yaxisStart, yaxisEnd);
        return ServerResponse.createBySuccess(ewbProbabilityPlotVos);
    }
}
